@@ -20,7 +20,7 @@
 #include "benchmark/benchmark.h"
 
 #include "./benchmark_util.h"
-#include "graphar/graph_info.h"
+#include "graphar/api/info.h"
 
 namespace graphar {
 
