@@ -24,12 +24,9 @@
 
 #include "./util.h"
 
-#include "graphar/graph_info.h"
-#include "graphar/util/adj_list_type.h"
-#include "graphar/util/data_type.h"
-#include "graphar/util/file_type.h"
-#include "graphar/util/filesystem.h"
-#include "graphar/util/version_parser.h"
+#include "graphar/api/info.h"
+#include "graphar/fwd.h"
+#include "graphar/status.h"
 
 #include <catch2/catch_test_macros.hpp>
 
@@ -80,9 +77,9 @@ TEST_CASE_METHOD(GlobalFixture, "Property") {
 
 TEST_CASE_METHOD(GlobalFixture, "PropertyGroup") {
   Property p0("p0", int32(), true);
-  Property p1("p1", int32(), false);
-  Property p2("p2", string(), false);
-  Property p3("p3", float32(), false);
+  Property p1("p1", int32(), false, true, Cardinality::SINGLE);
+  Property p2("p2", string(), false, true, Cardinality::LIST);
+  Property p3("p3", float32(), false, true, Cardinality::SET);
   Property p4("p4", float64(), false);
 
   PropertyGroup pg0({p0, p1}, FileType::CSV, "p0_and_p1/");
@@ -99,6 +96,11 @@ TEST_CASE_METHOD(GlobalFixture, "PropertyGroup") {
     REQUIRE(p.type->ToTypeName() == int32()->ToTypeName());
     REQUIRE(p.is_primary == true);
     REQUIRE(p.is_nullable == false);
+    // cardinality
+    REQUIRE(p0.cardinality == Cardinality::SINGLE);
+    REQUIRE(p1.cardinality == Cardinality::SINGLE);
+    REQUIRE(p2.cardinality == Cardinality::LIST);
+    REQUIRE(p3.cardinality == Cardinality::SET);
   }
 
   SECTION("FileType") {
@@ -181,17 +183,19 @@ TEST_CASE_METHOD(GlobalFixture, "AdjacentList") {
 }
 
 TEST_CASE_METHOD(GlobalFixture, "VertexInfo") {
-  std::string label = "test_vertex";
+  std::string type = "test_vertex";
   int chunk_size = 100;
   auto version = std::make_shared<InfoVersion>(1);
   auto pg = CreatePropertyGroup(
-      {Property("p0", int32(), true), Property("p1", string(), false)},
-      FileType::CSV, "p0_p1/");
+      {Property("p0", int32(), true),
+       Property("p1", string(), false, true, Cardinality::LIST),
+       Property("p2", string(), false, true, Cardinality::SET)},
+      FileType::PARQUET, "p0_p1/");
   auto vertex_info =
-      CreateVertexInfo(label, chunk_size, {pg}, "test_vertex", version);
+      CreateVertexInfo(type, chunk_size, {pg}, {}, "test_vertex", version);
 
   SECTION("Basics") {
-    REQUIRE(vertex_info->GetLabel() == label);
+    REQUIRE(vertex_info->GetType() == type);
     REQUIRE(vertex_info->GetChunkSize() == chunk_size);
     REQUIRE(vertex_info->GetPrefix() == "test_vertex");
     REQUIRE(vertex_info->version()->ToString() == "gar/v1");
@@ -211,6 +215,12 @@ TEST_CASE_METHOD(GlobalFixture, "VertexInfo") {
     REQUIRE(vertex_info->IsPrimaryKey("p1") == false);
     REQUIRE(vertex_info->IsNullableKey("p0") == false);
     REQUIRE(vertex_info->IsNullableKey("p1") == true);
+    REQUIRE(vertex_info->GetPropertyCardinality("p0").value() ==
+            Cardinality::SINGLE);
+    REQUIRE(vertex_info->GetPropertyCardinality("p1").value() ==
+            Cardinality::LIST);
+    REQUIRE(vertex_info->GetPropertyCardinality("p2").value() ==
+            Cardinality::SET);
     REQUIRE(vertex_info->HasProperty("not_exist") == false);
     REQUIRE(vertex_info->IsPrimaryKey("not_exist") == false);
     REQUIRE(vertex_info->HasPropertyGroup(nullptr) == false);
@@ -226,27 +236,34 @@ TEST_CASE_METHOD(GlobalFixture, "VertexInfo") {
 
   SECTION("IsValidate") {
     REQUIRE(vertex_info->IsValidated() == true);
-    auto invalid_pg =
-        CreatePropertyGroup({Property("p0", nullptr, true)}, FileType::CSV);
-    auto invalid_vertex_info0 = CreateVertexInfo(
-        label, chunk_size, {invalid_pg}, "test_vertex/", version);
+    auto invalid_pg = CreatePropertyGroup(
+        {Property("p0", list(string()), true, false, Cardinality::SET)},
+        FileType::CSV);
+    auto invalid_vertex_info0 = CreateVertexInfo(type, chunk_size, {invalid_pg},
+                                                 {}, "test_vertex/", version);
     REQUIRE(invalid_vertex_info0->IsValidated() == false);
-    VertexInfo invalid_vertex_info1("", chunk_size, {pg}, "test_vertex/",
+    invalid_pg =
+        CreatePropertyGroup({Property("p0", nullptr, true)}, FileType::CSV);
+    invalid_vertex_info0 = CreateVertexInfo(type, chunk_size, {invalid_pg}, {},
+                                            "test_vertex/", version);
+    REQUIRE(invalid_vertex_info0->IsValidated() == false);
+    VertexInfo invalid_vertex_info1("", chunk_size, {pg}, {}, "test_vertex/",
                                     version);
     REQUIRE(invalid_vertex_info1.IsValidated() == false);
-    VertexInfo invalid_vertex_info2(label, 0, {pg}, "test_vertex/", version);
+    VertexInfo invalid_vertex_info2(type, 0, {pg}, {}, "test_vertex/", version);
     REQUIRE(invalid_vertex_info2.IsValidated() == false);
     // check if prefix empty
     auto vertex_info_empty_prefix =
-        CreateVertexInfo(label, chunk_size, {pg}, "", version);
+        CreateVertexInfo(type, chunk_size, {pg}, {}, "", version);
     REQUIRE(vertex_info_empty_prefix->IsValidated() == true);
   }
 
   SECTION("CreateVertexInfo") {
-    auto vertex_info3 = CreateVertexInfo("", chunk_size, {pg}, "test_vertex/");
+    auto vertex_info3 =
+        CreateVertexInfo("", chunk_size, {pg}, {}, "test_vertex/");
     REQUIRE(vertex_info3 == nullptr);
 
-    auto vertex_info4 = CreateVertexInfo(label, 0, {pg}, "test_vertex/");
+    auto vertex_info4 = CreateVertexInfo(type, 0, {pg}, {}, "test_vertex/");
     REQUIRE(vertex_info4 == nullptr);
   }
 
@@ -254,57 +271,64 @@ TEST_CASE_METHOD(GlobalFixture, "VertexInfo") {
     auto dump_result = vertex_info->Dump();
     REQUIRE(dump_result.status().ok());
     std::string expected = R"(chunk_size: 100
-label: test_vertex
 prefix: test_vertex
 property_groups: 
-  - file_type: csv
+  - file_type: parquet
     prefix: p0_p1/
     properties: 
       - data_type: int32
         is_nullable: false
         is_primary: true
         name: p0
-      - data_type: string
+      - cardinality: list
+        data_type: string
         is_nullable: true
         is_primary: false
         name: p1
+      - cardinality: set
+        data_type: string
+        is_nullable: true
+        is_primary: false
+        name: p2
+type: test_vertex
 version: gar/v1
 )";
     REQUIRE(dump_result.value() == expected);
     auto vertex_info_empty_version =
-        CreateVertexInfo(label, chunk_size, {pg}, "test_vertex/");
+        CreateVertexInfo(type, chunk_size, {pg}, {}, "test_vertex/");
     REQUIRE(vertex_info_empty_version->Dump().status().ok());
   }
 
   SECTION("Save") {
     // save to a simple output path
-    REQUIRE(vertex_info->Save("/tmp/" + label + ".vertex.yml").ok());
+    REQUIRE(vertex_info->Save("/tmp/" + type + ".vertex.yml").ok());
     // save to a URI path
-    REQUIRE(vertex_info->Save("file:///tmp/" + label + ".vertex.yml").ok());
+    REQUIRE(vertex_info->Save("file:///tmp/" + type + ".vertex.yml").ok());
   }
 
   SECTION("AddPropertyGroup") {
-    auto pg2 = CreatePropertyGroup({Property("p2", int32(), false)},
-                                   FileType::CSV, "p2/");
-    auto maybe_extend_info = vertex_info->AddPropertyGroup(pg2);
+    auto pg3 = CreatePropertyGroup({Property("p3", int32(), false)},
+                                   FileType::CSV, "p3/");
+    auto maybe_extend_info = vertex_info->AddPropertyGroup(pg3);
     REQUIRE(maybe_extend_info.status().ok());
     auto extend_info = maybe_extend_info.value();
     REQUIRE(extend_info->PropertyGroupNum() == 2);
     REQUIRE(extend_info->HasProperty("p2") == true);
-    REQUIRE(extend_info->HasPropertyGroup(pg2) == true);
+    REQUIRE(extend_info->HasPropertyGroup(pg3) == true);
     REQUIRE(extend_info->GetPropertyGroups().size() == 2);
-    REQUIRE(*(extend_info->GetPropertyGroups()[1]) == *pg2);
-    REQUIRE(extend_info->GetPropertyType("p2").value()->ToTypeName() ==
+    REQUIRE(*(extend_info->GetPropertyGroups()[1]) == *pg3);
+    REQUIRE(extend_info->GetPropertyType("p3").value()->ToTypeName() ==
             int32()->ToTypeName());
-    REQUIRE(extend_info->IsPrimaryKey("p2") == false);
-    REQUIRE(extend_info->IsNullableKey("p2") == true);
-    auto extend_info2 = extend_info->AddPropertyGroup(pg2);
-    REQUIRE(!extend_info2.status().ok());
+    REQUIRE(extend_info->IsPrimaryKey("p3") == false);
+    REQUIRE(extend_info->IsNullableKey("p3") == true);
+    REQUIRE(extend_info->GetPropertyCardinality("p3") == Cardinality::SINGLE);
+    auto extend_info3 = extend_info->AddPropertyGroup(pg3);
+    REQUIRE(!extend_info3.status().ok());
   }
 }
 
 TEST_CASE_METHOD(GlobalFixture, "EdgeInfo") {
-  std::string src_label = "person", edge_label = "knows", dst_label = "person";
+  std::string src_type = "person", edge_type = "knows", dst_type = "person";
   int chunk_size = 1024;
   int src_chunk_size = 100;
   int dst_chunk_size = 100;
@@ -316,14 +340,14 @@ TEST_CASE_METHOD(GlobalFixture, "EdgeInfo") {
   auto pg = CreatePropertyGroup(
       {Property("p0", int32(), true), Property("p1", string(), false)},
       FileType::CSV, "p0_p1/");
-  auto edge_info = CreateEdgeInfo(src_label, edge_label, dst_label, chunk_size,
+  auto edge_info = CreateEdgeInfo(src_type, edge_type, dst_type, chunk_size,
                                   src_chunk_size, dst_chunk_size, directed,
                                   {adj_list}, {pg}, "test_edge/", version);
 
   SECTION("Basics") {
-    REQUIRE(edge_info->GetSrcLabel() == src_label);
-    REQUIRE(edge_info->GetEdgeLabel() == edge_label);
-    REQUIRE(edge_info->GetDstLabel() == dst_label);
+    REQUIRE(edge_info->GetSrcType() == src_type);
+    REQUIRE(edge_info->GetEdgeType() == edge_type);
+    REQUIRE(edge_info->GetDstType() == dst_type);
     REQUIRE(edge_info->GetChunkSize() == chunk_size);
     REQUIRE(edge_info->GetSrcChunkSize() == src_chunk_size);
     REQUIRE(edge_info->GetDstChunkSize() == dst_chunk_size);
@@ -388,14 +412,23 @@ TEST_CASE_METHOD(GlobalFixture, "EdgeInfo") {
     auto invalid_pg =
         CreatePropertyGroup({Property("p0", nullptr, true)}, FileType::CSV);
     auto invalid_edge_info0 =
-        CreateEdgeInfo(src_label, edge_label, dst_label, chunk_size,
+        CreateEdgeInfo(src_type, edge_type, dst_type, chunk_size,
+                       src_chunk_size, dst_chunk_size, directed, {adj_list},
+                       {invalid_pg}, "test_edge/", version);
+    REQUIRE(invalid_edge_info0->IsValidated() == false);
+    // edgeInfo does not support list/set cardinality
+    invalid_pg = CreatePropertyGroup(
+        {Property("p_cardinality", string(), false, true, Cardinality::LIST)},
+        FileType::PARQUET);
+    auto cardinality_invalid_edge_info0 =
+        CreateEdgeInfo(src_type, edge_type, dst_type, chunk_size,
                        src_chunk_size, dst_chunk_size, directed, {adj_list},
                        {invalid_pg}, "test_edge/", version);
     REQUIRE(invalid_edge_info0->IsValidated() == false);
     for (int i = 0; i < 3; i++) {
-      std::vector<std::string> labels = {src_label, edge_label, dst_label};
-      labels[i] = "";
-      EdgeInfo invalid_edge_info1(labels[0], labels[1], labels[2], chunk_size,
+      std::vector<std::string> types = {src_type, edge_type, dst_type};
+      types[i] = "";
+      EdgeInfo invalid_edge_info1(types[0], types[1], types[2], chunk_size,
                                   src_chunk_size, dst_chunk_size, directed,
                                   {adj_list}, {pg}, "test_edge/", version);
       REQUIRE(invalid_edge_info1.IsValidated() == false);
@@ -403,7 +436,7 @@ TEST_CASE_METHOD(GlobalFixture, "EdgeInfo") {
     for (int i = 0; i < 3; i++) {
       std::vector<int> sizes = {chunk_size, src_chunk_size, dst_chunk_size};
       sizes[i] = 0;
-      EdgeInfo invalid_edge_info2(src_label, edge_label, dst_label, sizes[0],
+      EdgeInfo invalid_edge_info2(src_type, edge_type, dst_type, sizes[0],
                                   sizes[1], sizes[2], directed, {adj_list},
                                   {pg}, "test_edge/", version);
       REQUIRE(invalid_edge_info2.IsValidated() == false);
@@ -411,30 +444,30 @@ TEST_CASE_METHOD(GlobalFixture, "EdgeInfo") {
 
     // check if prefix empty
     auto edge_info_with_empty_prefix = CreateEdgeInfo(
-        src_label, edge_label, dst_label, chunk_size, src_chunk_size,
+        src_type, edge_type, dst_type, chunk_size, src_chunk_size,
         dst_chunk_size, directed, {adj_list}, {pg}, "", version);
     REQUIRE(edge_info_with_empty_prefix->IsValidated() == true);
   }
 
   SECTION("CreateEdgeInfo") {
     for (int i = 0; i < 3; i++) {
-      std::vector<std::string> labels = {src_label, edge_label, dst_label};
-      labels[i] = "";
-      auto edge_info = CreateEdgeInfo(
-          labels[0], labels[1], labels[2], chunk_size, src_chunk_size,
-          dst_chunk_size, directed, {adj_list}, {pg}, "test_edge/", version);
+      std::vector<std::string> types = {src_type, edge_type, dst_type};
+      types[i] = "";
+      auto edge_info = CreateEdgeInfo(types[0], types[1], types[2], chunk_size,
+                                      src_chunk_size, dst_chunk_size, directed,
+                                      {adj_list}, {pg}, "test_edge/", version);
       REQUIRE(edge_info == nullptr);
     }
     for (int i = 0; i < 3; i++) {
       std::vector<int> sizes = {chunk_size, src_chunk_size, dst_chunk_size};
       sizes[i] = 0;
-      auto edge_info = CreateEdgeInfo(src_label, edge_label, dst_label,
-                                      sizes[0], sizes[1], sizes[2], directed,
-                                      {adj_list}, {pg}, "test_edge/", version);
+      auto edge_info = CreateEdgeInfo(src_type, edge_type, dst_type, sizes[0],
+                                      sizes[1], sizes[2], directed, {adj_list},
+                                      {pg}, "test_edge/", version);
       REQUIRE(edge_info == nullptr);
     }
     auto edge_info_empty_adjlist = CreateEdgeInfo(
-        src_label, edge_label, dst_label, chunk_size, src_chunk_size,
+        src_type, edge_type, dst_type, chunk_size, src_chunk_size,
         dst_chunk_size, directed, {}, {pg}, "test_edge/");
     REQUIRE(edge_info_empty_adjlist == nullptr);
   }
@@ -450,8 +483,8 @@ TEST_CASE_METHOD(GlobalFixture, "EdgeInfo") {
 chunk_size: 1024
 directed: true
 dst_chunk_size: 100
-dst_label: person
-edge_label: knows
+dst_type: person
+edge_type: knows
 prefix: test_edge/
 property_groups: 
   - file_type: csv
@@ -466,21 +499,21 @@ property_groups:
         is_primary: false
         name: p1
 src_chunk_size: 100
-src_label: person
+src_type: person
 version: gar/v1
 )";
     REQUIRE(dump_result.value() == expected);
     auto edge_info_empty_version = CreateEdgeInfo(
-        src_label, edge_label, dst_label, chunk_size, src_chunk_size,
+        src_type, edge_type, dst_type, chunk_size, src_chunk_size,
         dst_chunk_size, directed, {adj_list}, {pg});
     REQUIRE(edge_info_empty_version->Dump().status().ok());
   }
 
   SECTION("Save") {
     // save to a simple output path
-    REQUIRE(edge_info->Save("/tmp/" + edge_label + ".edge.yml").ok());
+    REQUIRE(edge_info->Save("/tmp/" + edge_type + ".edge.yml").ok());
     // save to a URI path
-    REQUIRE(edge_info->Save("file:///tmp/" + edge_label + ".edge.yml").ok());
+    REQUIRE(edge_info->Save("file:///tmp/" + edge_type + ".edge.yml").ok());
   }
 
   SECTION("AddAdjacentList") {
@@ -526,7 +559,7 @@ TEST_CASE_METHOD(GlobalFixture, "GraphInfo") {
       {Property("p0", int32(), true), Property("p1", string(), false)},
       FileType::CSV, "p0_p1/");
   auto vertex_info =
-      CreateVertexInfo("test_vertex", 100, {pg}, "test_vertex/", version);
+      CreateVertexInfo("test_vertex", 100, {pg}, {}, "test_vertex/", version);
   std::unordered_map<std::string, std::string> extra_info = {
       {"category", "test graph"}};
   auto edge_info =
@@ -534,7 +567,7 @@ TEST_CASE_METHOD(GlobalFixture, "GraphInfo") {
                      {CreateAdjacentList(AdjListType::ordered_by_source,
                                          FileType::CSV, "adj_list/")},
                      {pg}, "test_edge/", version);
-  auto graph_info = CreateGraphInfo(name, {vertex_info}, {edge_info},
+  auto graph_info = CreateGraphInfo(name, {vertex_info}, {edge_info}, {},
                                     "test_graph/", version, extra_info);
 
   SECTION("Basics") {
@@ -549,7 +582,7 @@ TEST_CASE_METHOD(GlobalFixture, "GraphInfo") {
 
   SECTION("ExtraInfo") {
     auto graph_info_with_extra_info =
-        CreateGraphInfo(name, {vertex_info}, {edge_info}, "test_graph/",
+        CreateGraphInfo(name, {vertex_info}, {edge_info}, {}, "test_graph/",
                         version, {{"key1", "value1"}, {"key2", "value2"}});
     const auto& extra_info = graph_info_with_extra_info->GetExtraInfo();
     REQUIRE(extra_info.size() == 2);
@@ -561,33 +594,33 @@ TEST_CASE_METHOD(GlobalFixture, "GraphInfo") {
 
   SECTION("VertexInfo") {
     REQUIRE(graph_info->VertexInfoNum() == 1);
-    REQUIRE(graph_info->GetVertexInfoByIndex(0)->GetLabel() == "test_vertex");
+    REQUIRE(graph_info->GetVertexInfoByIndex(0)->GetType() == "test_vertex");
     REQUIRE(graph_info->GetVertexInfoByIndex(1) == nullptr);
-    REQUIRE(graph_info->GetVertexInfo("test_vertex")->GetLabel() ==
+    REQUIRE(graph_info->GetVertexInfo("test_vertex")->GetType() ==
             "test_vertex");
     REQUIRE(graph_info->GetVertexInfo("not_exist") == nullptr);
     REQUIRE(graph_info->GetVertexInfos().size() == 1);
-    REQUIRE(graph_info->GetVertexInfos()[0]->GetLabel() == "test_vertex");
+    REQUIRE(graph_info->GetVertexInfos()[0]->GetType() == "test_vertex");
   }
 
   SECTION("EdgeInfo") {
     REQUIRE(graph_info->EdgeInfoNum() == 1);
-    REQUIRE(graph_info->GetEdgeInfoByIndex(0)->GetEdgeLabel() == "knows");
+    REQUIRE(graph_info->GetEdgeInfoByIndex(0)->GetEdgeType() == "knows");
     REQUIRE(graph_info->GetEdgeInfoByIndex(1) == nullptr);
     REQUIRE(
-        graph_info->GetEdgeInfo("person", "knows", "person")->GetEdgeLabel() ==
+        graph_info->GetEdgeInfo("person", "knows", "person")->GetEdgeType() ==
         "knows");
     REQUIRE(graph_info->GetEdgeInfo("not_exist", "knows", "person") == nullptr);
     REQUIRE(graph_info->GetEdgeInfos().size() == 1);
-    REQUIRE(graph_info->GetEdgeInfos()[0]->GetEdgeLabel() == "knows");
+    REQUIRE(graph_info->GetEdgeInfos()[0]->GetEdgeType() == "knows");
   }
 
   SECTION("IsValidated") {
     REQUIRE(graph_info->IsValidated() == true);
     auto invalid_vertex_info =
-        CreateVertexInfo("", 100, {pg}, "test_vertex/", version);
+        CreateVertexInfo("", 100, {pg}, {}, "test_vertex/", version);
     auto invalid_graph_info0 = CreateGraphInfo(
-        name, {invalid_vertex_info}, {edge_info}, "test_graph/", version);
+        name, {invalid_vertex_info}, {edge_info}, {}, "test_graph/", version);
     REQUIRE(invalid_graph_info0->IsValidated() == false);
     auto invalid_edge_info =
         CreateEdgeInfo("", "knows", "person", 1024, 100, 100, true,
@@ -595,23 +628,23 @@ TEST_CASE_METHOD(GlobalFixture, "GraphInfo") {
                                            FileType::CSV, "adj_list/")},
                        {pg}, "test_edge/", version);
     auto invalid_graph_info1 = CreateGraphInfo(
-        name, {vertex_info}, {invalid_edge_info}, "test_graph/", version);
+        name, {vertex_info}, {invalid_edge_info}, {}, "test_graph/", version);
     REQUIRE(invalid_graph_info1->IsValidated() == false);
-    GraphInfo invalid_graph_info2("", {vertex_info}, {edge_info}, "test_graph/",
-                                  version);
+    GraphInfo invalid_graph_info2("", {vertex_info}, {edge_info}, {},
+                                  "test_graph/", version);
     REQUIRE(invalid_graph_info2.IsValidated() == false);
-    GraphInfo invalid_graph_info3(name, {vertex_info}, {edge_info}, "",
+    GraphInfo invalid_graph_info3(name, {vertex_info}, {edge_info}, {}, "",
                                   version);
     REQUIRE(invalid_graph_info3.IsValidated() == false);
     // check if prefix empty, graph_info with empty prefix is invalid
     auto graph_info_with_empty_prefix =
-        CreateGraphInfo(name, {vertex_info}, {edge_info}, "", version);
+        CreateGraphInfo(name, {vertex_info}, {edge_info}, {}, "", version);
     REQUIRE(graph_info_with_empty_prefix->IsValidated() == false);
   }
 
   SECTION("CreateGraphInfo") {
     auto graph_info_empty_name =
-        CreateGraphInfo("", {vertex_info}, {edge_info}, "test_graph/");
+        CreateGraphInfo("", {vertex_info}, {edge_info}, {}, "test_graph/");
     REQUIRE(graph_info_empty_name == nullptr);
   }
 
@@ -631,7 +664,7 @@ vertices:
 )";
     REQUIRE(dump_result.value() == expected);
     auto graph_info_empty_version =
-        CreateGraphInfo(name, {vertex_info}, {edge_info}, "test_graph/");
+        CreateGraphInfo(name, {vertex_info}, {edge_info}, {}, "test_graph/");
     REQUIRE(graph_info_empty_version->Dump().status().ok());
   }
 
@@ -643,19 +676,19 @@ vertices:
   }
 
   SECTION("AddVertex") {
-    auto vertex_info2 =
-        CreateVertexInfo("test_vertex2", 100, {pg}, "test_vertex2/", version);
+    auto vertex_info2 = CreateVertexInfo("test_vertex2", 100, {pg}, {},
+                                         "test_vertex2/", version);
     auto maybe_extend_info = graph_info->AddVertex(vertex_info2);
     REQUIRE(maybe_extend_info.status().ok());
     auto extend_info = maybe_extend_info.value();
     REQUIRE(extend_info->VertexInfoNum() == 2);
-    REQUIRE(extend_info->GetVertexInfoByIndex(1)->GetLabel() == "test_vertex2");
+    REQUIRE(extend_info->GetVertexInfoByIndex(1)->GetType() == "test_vertex2");
     REQUIRE(extend_info->GetVertexInfoByIndex(2) == nullptr);
-    REQUIRE(extend_info->GetVertexInfo("test_vertex2")->GetLabel() ==
+    REQUIRE(extend_info->GetVertexInfo("test_vertex2")->GetType() ==
             "test_vertex2");
     REQUIRE(extend_info->GetVertexInfo("not_exist") == nullptr);
     REQUIRE(extend_info->GetVertexInfos().size() == 2);
-    REQUIRE(extend_info->GetVertexInfos()[1]->GetLabel() == "test_vertex2");
+    REQUIRE(extend_info->GetVertexInfos()[1]->GetType() == "test_vertex2");
     auto extend_info2 = extend_info->AddVertex(vertex_info2);
     REQUIRE(!extend_info2.status().ok());
   }
@@ -670,21 +703,22 @@ vertices:
     REQUIRE(maybe_extend_info.status().ok());
     auto extend_info = maybe_extend_info.value();
     REQUIRE(extend_info->EdgeInfoNum() == 2);
-    REQUIRE(extend_info->GetEdgeInfoByIndex(1)->GetEdgeLabel() == "knows2");
+    REQUIRE(extend_info->GetEdgeInfoByIndex(1)->GetEdgeType() == "knows2");
     REQUIRE(extend_info->GetEdgeInfoByIndex(2) == nullptr);
-    REQUIRE(extend_info->GetEdgeInfo("person", "knows2", "person")
-                ->GetEdgeLabel() == "knows2");
+    REQUIRE(
+        extend_info->GetEdgeInfo("person", "knows2", "person")->GetEdgeType() ==
+        "knows2");
     REQUIRE(extend_info->GetEdgeInfo("not_exist", "knows2", "person") ==
             nullptr);
     REQUIRE(extend_info->GetEdgeInfos().size() == 2);
-    REQUIRE(extend_info->GetEdgeInfos()[1]->GetEdgeLabel() == "knows2");
+    REQUIRE(extend_info->GetEdgeInfos()[1]->GetEdgeType() == "knows2");
     auto extend_info2 = extend_info->AddEdge(edge_info2);
     REQUIRE(!extend_info2.status().ok());
   }
 }
 
 TEST_CASE_METHOD(GlobalFixture, "LoadFromYaml") {
-  std::string vertex_info_yaml = R"(label: person
+  std::string vertex_info_yaml = R"(type: person
 chunk_size: 100
 prefix: vertex/person/
 property_groups:
@@ -694,6 +728,13 @@ property_groups:
         is_primary: true
         is_nullable: false
     file_type: parquet
+  - properties:
+      - name: email
+        data_type: string
+        is_primary: false
+        is_nullable: true
+        cardinality: list
+    file_type: parquet
   - properties:
       - name: firstName
         data_type: string
@@ -710,9 +751,9 @@ property_groups:
     file_type: parquet
 version: gar/v1
 )";
-  std::string edge_info_yaml = R"(src_label: person
-edge_label: knows
-dst_label: person
+  std::string edge_info_yaml = R"(src_type: person
+edge_type: knows
+dst_type: person
 chunk_size: 1024
 src_chunk_size: 100
 dst_chunk_size: 100
@@ -749,19 +790,53 @@ extra_info:
     auto maybe_vertex_info = VertexInfo::Load(vertex_info_yaml);
     REQUIRE(!maybe_vertex_info.has_error());
     auto vertex_info = maybe_vertex_info.value();
-    REQUIRE(vertex_info->GetLabel() == "person");
+    REQUIRE(vertex_info->GetType() == "person");
     REQUIRE(vertex_info->GetChunkSize() == 100);
     REQUIRE(vertex_info->GetPrefix() == "vertex/person/");
     REQUIRE(vertex_info->version()->ToString() == "gar/v1");
+    REQUIRE(vertex_info->GetPropertyCardinality("id").value() ==
+            Cardinality::SINGLE);
+    REQUIRE(vertex_info->GetPropertyCardinality("email").value() ==
+            Cardinality::LIST);
   }
 
   SECTION("EdgeInfo::Load") {
     auto maybe_edge_info = EdgeInfo::Load(edge_info_yaml);
     REQUIRE(!maybe_edge_info.has_error());
     auto edge_info = maybe_edge_info.value();
-    REQUIRE(edge_info->GetSrcLabel() == "person");
-    REQUIRE(edge_info->GetEdgeLabel() == "knows");
-    REQUIRE(edge_info->GetDstLabel() == "person");
+    REQUIRE(edge_info->GetSrcType() == "person");
+    REQUIRE(edge_info->GetEdgeType() == "knows");
+    REQUIRE(edge_info->GetDstType() == "person");
+  }
+
+  edge_info_yaml = R"(src_type: person
+edge_type: knows
+dst_type: person
+chunk_size: 1024
+src_chunk_size: 100
+dst_chunk_size: 100
+directed: false
+prefix: edge/person_knows_person/
+adj_lists:
+  - ordered: false
+    aligned_by: src
+    file_type: parquet
+property_groups:
+  - file_type: parquet
+    properties:
+      - name: creationDate
+        data_type: string
+        is_primary: false
+        is_nullable: true
+        cardinality: list
+version: gar/v1
+)";
+
+  SECTION("EdgeInfo::Load with cardinality") {
+    auto maybe_edge_info = EdgeInfo::Load(edge_info_yaml);
+    REQUIRE(maybe_edge_info.has_error());
+    REQUIRE(maybe_edge_info.status().code() == StatusCode::kYamlError);
+    std::cout << maybe_edge_info.status().message() << std::endl;
   }
 
   SECTION("GraphInfo::Load") {
@@ -779,18 +854,28 @@ extra_info:
   }
 }
 
-TEST_CASE_METHOD(GlobalFixture, "LoadFromS3", "[.hidden]") {
-  std::string path =
-      "s3://graphar/ldbc/ldbc.graph.yml"
-      "?endpoint_override=graphscope.oss-cn-beijing.aliyuncs.com";
-  auto graph_info_result = GraphInfo::Load(path);
-  std::cout << graph_info_result.status().message() << std::endl;
-  REQUIRE(!graph_info_result.has_error());
-  auto graph_info = graph_info_result.value();
-  REQUIRE(graph_info->GetName() == "ldbc");
-  const auto& vertex_infos = graph_info->GetVertexInfos();
-  const auto& edge_infos = graph_info->GetEdgeInfos();
-  REQUIRE(vertex_infos.size() == 8);
-  REQUIRE(edge_infos.size() == 23);
-}
+/*
+TODO(acezen): need to mock S3 server to test this case, this private
+service is not available for public access.
+*/
+// TEST_CASE_METHOD(GlobalFixture, "LoadFromS3") {
+//   // explicitly call InitS3 to initialize S3 APIs before using
+//   // S3 file system.
+//   InitializeS3();
+//   std::string path =
+//       "s3://graphar/ldbc/ldbc.graph.yml"
+//       "?endpoint_override=graphscope.oss-cn-beijing.aliyuncs.com";
+//   auto graph_info_result = GraphInfo::Load(path);
+//   std::cout << graph_info_result.status().message() << std::endl;
+//   REQUIRE(!graph_info_result.has_error());
+//   auto graph_info = graph_info_result.value();
+//   REQUIRE(graph_info->GetName() == "ldbc");
+//   const auto& vertex_infos = graph_info->GetVertexInfos();
+//   const auto& edge_infos = graph_info->GetEdgeInfos();
+//   REQUIRE(vertex_infos.size() == 8);
+//   REQUIRE(edge_infos.size() == 23);
+//   // explicitly call FinalizeS3 to avoid memory leak
+//   FinalizeS3();
+// }
+
 }  // namespace graphar
