@@ -20,5 +20,5 @@
 package org.apache.graphar.util;
 
 public class JNILibraryName {
-    public static final String GAR_JNI_LIBRARY_NAME = "gar-jni";
+    public static final String GAR_JNI_LIBRARY_NAME = "graphar-jni";
 }
